@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 #
-# Copyright (C) 2011-2014  Leo Singer
+# Copyright (C) 2011-2017  Leo Singer
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +26,6 @@ Public-domain cartographic data is courtesy of Natural Earth
 (http://www.naturalearthdata.com) and processed with MapShaper
 (http://www.mapshaper.org).
 """
-__author__ = "Leo Singer <leo.singer@ligo.org>"
 
 
 # Command line interface
@@ -33,6 +33,9 @@ __author__ = "Leo Singer <leo.singer@ligo.org>"
 import argparse
 from lalinference.bayestar import command
 parser = command.ArgumentParser(parents=[command.figure_parser])
+parser.add_argument(
+    '--annotate', default=False, action='store_true',
+    help='annotate plot with information about the event')
 parser.add_argument(
     '--contour', metavar='PERCENT', type=float, nargs='+',
     help='plot contour enclosing this percentage of'
@@ -51,9 +54,6 @@ parser.add_argument(
     '--geo', action='store_true', default=False,
     help='Plot in geographic coordinates, (lat, lon) instead of (RA, Dec)'
     ' [default: %(default)s]')
-parser.add_argument(
-    '--transparent', action='store_true', default=False,
-    help='Save image with transparent background [default: %(default)s]')
 parser.add_argument(
     'input', metavar='INPUT.fits[.gz]', type=argparse.FileType('rb'),
     default='-', nargs='?', help='Input FITS file [default: stdin]')
@@ -80,12 +80,14 @@ skymap, metadata = fits.read_sky_map(opts.input.name, nest=None)
 nside = hp.npix2nside(len(skymap))
 
 if opts.geo:
-    dlon = -lal.GreenwichMeanSiderealTime(lal.LIGOTimeGPS(metadata['gps_time'])) % (2*np.pi)
+    dlon = -lal.GreenwichMeanSiderealTime(
+        lal.LIGOTimeGPS(metadata['gps_time'])) % (2*np.pi)
 else:
     dlon = 0
 
 # Convert sky map from probability to probability per square degree.
-probperdeg2 = skymap / hp.nside2pixarea(nside, degrees=True)
+deg2perpix = hp.nside2pixarea(nside, degrees=True)
+probperdeg2 = skymap / deg2perpix
 
 # Plot sky map.
 vmax = probperdeg2.max()
@@ -108,17 +110,17 @@ if opts.contour:
 
 # Add continents.
 if opts.geo:
-    geojson_filename = os.path.join(os.path.dirname(plot.__file__),
-        'ne_simplified_coastline.json')
+    geojson_filename = os.path.join(
+        os.path.dirname(plot.__file__), 'ne_simplified_coastline.json')
     with open(geojson_filename, 'r') as geojson_file:
         geojson = json.load(geojson_file)
     for shape in geojson['geometries']:
         verts = np.deg2rad(shape['coordinates'])
         plt.plot(verts[:, 0], verts[:, 1], color='0.5', linewidth=0.5)
 
-radecs = np.deg2rad(opts.radec)
+radecs = np.deg2rad(opts.radec).tolist()
 if opts.inj_database:
-    query = '''SELECT longitude, latitude FROM sim_inspiral AS si
+    query = '''SELECT DISTINCT longitude, latitude FROM sim_inspiral AS si
                INNER JOIN coinc_event_map AS cm1
                ON (si.simulation_id = cm1.event_id)
                INNER JOIN coinc_event_map AS cm2
@@ -136,17 +138,30 @@ for ra, dec in radecs:
         ra = plot.reference_angle(ra + dlon)
     else:
         ra = plot.wrapped_angle(ra + dlon)
-    ax.plot(ra, dec, '*', markerfacecolor='white', markeredgecolor='black', markersize=10)
+    ax.plot(
+        ra, dec, '*',
+        markerfacecolor='white', markeredgecolor='black', markersize=10)
 
-# If we are using a new enough version of matplotlib, then
-# add a white outline to all text to make it stand out from the background.
+# Add a white outline to all text to make it stand out from the background.
 plot.outline_text(ax)
 
-# Make transparent.
-if opts.transparent:
-    fig.patch.set_alpha(0.)
-    ax.patch.set_alpha(0.)
-    ax.set_alpha(0.)
+if opts.annotate:
+    text = []
+    try:
+        objid = metadata['objid']
+    except KeyError:
+        pass
+    else:
+        text.append('event ID: {}'.format(objid))
+    if opts.contour:
+        pp = np.round(opts.contour).astype(int)
+        ii = np.round(np.searchsorted(np.sort(cls), opts.contour) *
+                      deg2perpix).astype(int)
+        for i, p in zip(ii, pp):
+            # FIXME: use Unicode symbol instead of TeX '$^2$'
+            # because of broken fonts on Scientific Linux 7.
+            text.append(u'{:d}% area: {:d} deg²'.format(p, i, grouping=True))
+    ax.text(1, 1, '\n'.join(text), transform=ax.transAxes, ha='right')
 
 # Show or save output.
 opts.output()
