@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2013-2016  Leo Singer
+# Copyright (C) 2013-2017  Leo Singer
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -19,15 +19,16 @@
 Postprocessing utilities for HEALPix sky maps
 """
 from __future__ import division
-__author__ = "Leo Singer <leo.singer@ligo.org>"
 
 
 import numpy as np
 import healpy as hp
 import collections
-import itertools
 import lal
 import lalsimulation
+from scipy.interpolate import interp1d
+from . import distance
+from . import moc
 from ..healpix_tree import *
 
 
@@ -70,6 +71,15 @@ def count_modes(m, nest=False):
     return nmodes
 
 
+def count_modes_moc(uniq, i):
+    n = len(uniq)
+    mask = np.concatenate((np.ones(i + 1, dtype=bool),
+                           np.zeros(n - i - 1, dtype=bool)))
+    sky_map = np.rec.fromarrays((uniq, mask), names=('UNIQ', 'MASK'))
+    sky_map = moc.rasterize(sky_map)['MASK']
+    return count_modes(sky_map, nest=True)
+
+
 def indicator(n, i):
     """Create a binary array of length n that is True for every index that is in
     i and False for every other index. Named after the indicator function."""
@@ -79,22 +89,29 @@ def indicator(n, i):
 
 
 def cos_angle_distance(theta0, phi0, theta1, phi1):
-    """Cosine of angular separation in radians between two points on the unit sphere."""
-    cos_angle_distance = (np.cos(phi1 - phi0) * np.sin(theta0) * np.sin(theta1)
+    """Cosine of angular separation in radians between two points on the
+    unit sphere."""
+    cos_angle_distance = (
+        np.cos(phi1 - phi0) * np.sin(theta0) * np.sin(theta1)
         + np.cos(theta0) * np.cos(theta1))
     return np.clip(cos_angle_distance, -1, 1)
 
+
 def angle_distance(theta0, phi0, theta1, phi1):
     """Angular separation in radians between two points on the unit sphere."""
     return np.arccos(cos_angle_distance(theta0, phi0, theta1, phi1))
 
 
 # Class to hold return value of find_injection method
-FoundInjection = collections.namedtuple('FoundInjection',
-    'searched_area searched_prob offset searched_modes contour_areas area_probs contour_modes')
+FoundInjection = collections.namedtuple(
+    'FoundInjection',
+    'searched_area searched_prob offset searched_modes contour_areas '
+    'area_probs contour_modes searched_prob_dist contour_dists '
+    'searched_vol searched_prob_vol contour_vols')
 
 
-def find_injection(sky_map, true_ra, true_dec, contours=(), areas=(), modes=False, nest=False):
+def find_injection_moc(sky_map, true_ra=None, true_dec=None, true_dist=None,
+                       contours=(), areas=(), modes=False, nest=False):
     """
     Given a sky map and the true right ascension and declination (in radians),
     find the smallest area in deg^2 that would have to be searched to find the
@@ -104,70 +121,165 @@ def find_injection(sky_map, true_ra, true_dec, contours=(), areas=(), modes=Fals
     containing a given total probability.
     """
 
-    # Compute the HEALPix lateral resolution parameter for this sky map.
-    npix = len(sky_map)
-    nside = hp.npix2nside(npix)
-    deg2perpix = hp.nside2pixarea(nside, degrees=True)
+    if (true_ra is None) ^ (true_dec is None):
+        raise ValueError('Both true_ra and true_dec must be provided or None')
 
-    # Convert from ra, dec to conventional spherical polar coordinates.
-    true_theta = 0.5 * np.pi - true_dec
-    true_phi = true_ra
+    contours = np.asarray(contours)
 
-    # Find the HEALPix pixel index of the mode of the posterior and of the
-    # true sky location.
-    mode_pix = np.argmax(sky_map)
-    true_pix = hp.ang2pix(nside, true_theta, true_phi, nest=nest)
+    distmean = sky_map.meta.get('distmean', np.nan)
 
-    # Compute spherical polar coordinates of true location.
-    mode_theta, mode_phi = hp.pix2ang(nside, mode_pix, nest=nest)
+    # Sort the pixels by descending posterior probability.
+    sky_map = np.flipud(np.sort(sky_map, order='PROBDENSITY'))
 
-    # Sort the pixels in the sky map by descending posterior probability and
-    # form the cumulative sum.  Record the total value.
-    indices = np.argsort(sky_map)[::-1]
-    cum_sky_map = np.cumsum(sky_map[indices])
+    # Find the pixel that contains the injection.
+    order, ipix = moc.uniq2nest(sky_map['UNIQ'])
+    max_order = np.max(order)
+    max_nside = hp.order2nside(max_order)
+    max_ipix = ipix << np.uint64(2 * (max_order - order))
+    ipix = ipix.astype(np.int64)
+    max_ipix = max_ipix.astype(np.int64)
+    if true_ra is not None:
+        true_theta = 0.5 * np.pi - true_dec
+        true_phi = true_ra
+        true_pix = hp.ang2pix(max_nside, true_theta, true_phi, nest=True)
+        i = np.argsort(max_ipix)
+        true_idx = i[np.digitize(true_pix, max_ipix[i]) - 1]
 
-    # Find the index of the true location in the cumulative distribution.
-    idx = next((i for i, pix in enumerate(indices) if pix == true_pix))
+    # Find the angular offset between the mode and true locations.
+    mode_theta, mode_phi = hp.pix2ang(
+        hp.order2nside(order[0]), ipix[0].astype(np.int64), nest=True)
+    if true_ra is None:
+        offset = np.nan
+    else:
+        offset = np.rad2deg(
+            angle_distance(true_theta, true_phi, mode_theta, mode_phi))
 
-    # Find the smallest area that would have to be searched to find
-    # the true location. Note that 1 is added to the index because we want
-    # the **length** of the array up to and including the idx'th element,
-    # not the index itself.
-    searched_area = (idx + 1) * deg2perpix
+    # Calculate the cumulative area in deg2 and the cumulative probability.
+    dA = moc.uniq2pixarea(sky_map['UNIQ'])
+    dP = sky_map['PROBDENSITY'] * dA
+    prob = np.cumsum(dP)
+    area = np.cumsum(dA) * np.square(180 / np.pi)
 
-    # Find the smallest posterior mass that would have to be searched to find
-    # the true location.
-    searched_prob = cum_sky_map[idx]
+    # Construct linear interpolants to map between probability and area.
+    # This allows us to compute more accurate contour areas and probabilities
+    # under the approximation that the pixels have constant probability
+    # density.
+    prob_padded = np.concatenate(([0], prob))
+    area_padded = np.concatenate(([0], area))
+    # FIXME: we should use the assume_sorted=True argument below, but
+    # it was added in Scipy 0.14.0, and our Scientific Linux 7 clusters
+    # only have Scipy 0.12.1.
+    prob_for_area = interp1d(area_padded, prob_padded)
+    area_for_prob = interp1d(prob_padded, area_padded)
 
-    # Get the total number of pixels that lie inside each contour.
-    ipix = np.searchsorted(cum_sky_map, contours)
+    if true_ra is None:
+        searched_area = searched_prob = np.nan
+    else:
+        # Find the smallest area that would have to be searched to find
+        # the true location.
+        searched_area = area[true_idx]
+
+        # Find the smallest posterior mass that would have to be searched to find
+        # the true location.
+        searched_prob = prob[true_idx]
+
+    # Find the contours of the given credible levels.
+    contour_idxs = np.digitize(contours, prob) - 1
 
     # For each of the given confidence levels, compute the area of the
     # smallest region containing that probability.
-    contour_areas = (deg2perpix * (ipix + 1)).tolist()
+    contour_areas = area_for_prob(contours).tolist()
 
     # For each listed area, find the probability contained within the
     # smallest credible region of that area.
-    area_probs = cum_sky_map[
-        np.round(np.asarray(areas) / deg2perpix).astype(np.intp)].tolist()
-
-    # Find the angular offset between the mode and true locations.
-    offset = np.rad2deg(angle_distance(true_theta, true_phi,
-        mode_theta, mode_phi))
+    area_probs = prob_for_area(areas).tolist()
 
     if modes:
-        # Count up the number of modes in each of the given contours.
-        searched_modes = count_modes(indicator(npix, indices[:idx+1]), nest=nest)
-        contour_modes = [count_modes(indicator(npix, indices[:i+1]), nest=nest)
-            for i in ipix]
+        if true_ra is None:
+            searched_modes = np.nan
+        else:
+            # Count up the number of modes in each of the given contours.
+            searched_modes = count_modes_moc(sky_map['UNIQ'], true_idx)
+        contour_modes = [
+            count_modes_moc(sky_map['UNIQ'], i) for i in contour_idxs]
     else:
-        searched_modes = None
-        contour_modes = None
+        searched_modes = np.nan
+        contour_modes = np.nan
+
+    # Distance stats now...
+    if 'DISTMU' in sky_map.dtype.names:
+        probdensity = sky_map['PROBDENSITY']
+        mu = sky_map['DISTMU']
+        sigma = sky_map['DISTSIGMA']
+        norm = sky_map['DISTNORM']
+        args = (dP, mu, sigma, norm)
+        if true_dist is None:
+            searched_prob_dist = np.nan
+        else:
+            searched_prob_dist = distance.marginal_cdf(true_dist, *args)
+        # FIXME: old verisons of Numpy can't handle passing zero-length
+        # arrays to generalized ufuncs. Remove this workaround once LIGO
+        # Data Grid clusters provide a more modern version of Numpy.
+        if len(contours) == 0:
+            contour_dists = []
+        else:
+            lo, hi = distance.marginal_ppf(
+                np.row_stack((
+                    0.5 * (1 - contours),
+                    0.5 * (1 + contours)
+                )), *args)
+            contour_dists = (hi - lo).tolist()
+
+        # Set up distance grid.
+        n_r = 1000
+        max_r = max(6 * distmean, np.max(true_dist))
+        d_r = max_r / n_r
+        r = d_r * np.arange(1, n_r)
+
+        # Calculate volume of frustum-shaped voxels with distance centered on r
+        # and extending from (r - d_r) to (r + d_r).
+        dV = (np.square(r) + np.square(d_r) / 12) * d_r * dA.reshape(-1, 1)
+
+        # Calculate probability within each voxel.
+        dP = probdensity.reshape(-1, 1) * dV * np.exp(
+            -0.5 * np.square(
+                (r.reshape(1, -1) - mu.reshape(-1, 1)) / sigma.reshape(-1, 1)
+            )
+        ) * (norm / sigma).reshape(-1, 1) / np.sqrt(2 * np.pi)
+        dP[np.isnan(dP)] = 0  # Suppress invalid values
+
+        # Calculate probability density per unit volume.
+        dP_dV = dP / dV
+        i = np.flipud(np.argsort(dP_dV.ravel()))
+
+        P_flat = np.cumsum(dP.ravel()[i])
+        V_flat = np.cumsum(dV.ravel()[i])
+
+        contour_vols = interp1d(
+            P_flat, V_flat, bounds_error=False)(contours).tolist()
+        P = np.empty_like(P_flat)
+        V = np.empty_like(V_flat)
+        P[i] = P_flat
+        V[i] = V_flat
+        P = P.reshape(dP.shape)
+        V = V.reshape(dV.shape)
+        if true_dist is None:
+            searched_vol = searched_prob_vol = np.nan
+        else:
+            i_radec = true_idx
+            i_dist = np.digitize(true_dist, r) - 1
+            searched_prob_vol = P[i_radec, i_dist]
+            searched_vol = V[i_radec, i_dist]
+    else:
+        searched_vol = searched_prob_vol = searched_prob_dist = np.nan
+        contour_dists = [np.nan] * len(contours)
+        contour_vols = [np.nan] * len(contours)
 
     # Done.
     return FoundInjection(
         searched_area, searched_prob, offset, searched_modes, contour_areas,
-        area_probs, contour_modes)
+        area_probs, contour_modes, searched_prob_dist, contour_dists,
+        searched_vol, searched_prob_vol, contour_vols)
 
 
 def _norm(vertices):
@@ -222,7 +334,7 @@ def contour(m, levels, nest=False, degrees=False, simplify=True):
     # faces is an npix X 4 array mapping HEALPix faces to their vertices.
     # neighbors is an npix X 4 array mapping faces to their nearest neighbors.
     faces = np.ascontiguousarray(
-            np.rollaxis(hp.boundaries(nside, np.arange(npix), nest=nest), 2, 1))
+        np.rollaxis(hp.boundaries(nside, np.arange(npix), nest=nest), 2, 1))
     dtype = faces.dtype
     faces = faces.view(np.dtype((np.void, dtype.itemsize * 3)))
     vertices, faces = np.unique(faces.ravel(), return_inverse=True)
@@ -244,12 +356,14 @@ def contour(m, levels, nest=False, degrees=False, simplify=True):
             for ipix2 in ipix2:
                 # Determine if we have already considered this pair of faces.
                 new_face_pair = frozenset((ipix1, ipix2))
-                if new_face_pair in face_pairs: continue
+                if new_face_pair in face_pairs:
+                    continue
                 face_pairs.add(new_face_pair)
 
                 # Determine if this pair of faces are on a boundary of the
                 # credible level.
-                if indicator[ipix1] == indicator[ipix2]: continue
+                if indicator[ipix1] == indicator[ipix2]:
+                    continue
 
                 # Add all common edges of this pair of faces.
                 i1 = np.concatenate((faces[ipix1], [faces[ipix1][0]]))
@@ -262,7 +376,8 @@ def contour(m, levels, nest=False, degrees=False, simplify=True):
 
         # Record a closed path for each cycle in the graph.
         cycles = [
-            np.take(vertices, cycle, axis=0) for cycle in nx.cycle_basis(graph)]
+            np.take(vertices, cycle, axis=0)
+            for cycle in nx.cycle_basis(graph)]
 
         # Simplify paths if requested
         if simplify:
@@ -470,8 +585,9 @@ def polar_profile(m, nest=False):
         m = hp.reorder(m, n2r=True)
 
     theta = np.arccos(costheta)
-    m_int = np.asarray([m[i:i+j].sum() * stheta * 0.5 * npix / j
-        for i, j, stheta in zip(startpix, ringpix, sintheta)])
+    m_int = np.asarray(
+        [m[i:i+j].sum() * stheta * 0.5 * npix / j
+         for i, j, stheta in zip(startpix, ringpix, sintheta)])
 
     return theta, m_int
 
