@@ -14,8 +14,10 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
+from . import events
 from . import fits
 from . import hdf5
+from .events import *
 from .fits import *
 from .hdf5 import *
-__all__ = fits.__all__ + hdf5.__all__
+__all__ = events.__all__ + fits.__all__ + hdf5.__all__
