@@ -162,35 +162,6 @@ def plot_grid_cells(cells, color, axis1=0, axis2=1):
 
         ax.add_patch(Rectangle((cell._bounds[axis1][0], cell._bounds[axis2][0]), ext1, ext2, edgecolor = color, facecolor='none'))
 
-#
-# Option parsing
-#
-
-def parse_param(popts):
-    """
-    Parse out the specification of the intrinsic space. Examples:
-
-    >>> parse_param(["mass1=1.4", "mass2", "spin1z=-1.0,10"])
-    {'mass1': 1.4, 'mass2': None, 'spin1z': (-1.0, 10.0)}
-    """
-    if popts is None:
-        return {}, {}
-    intr_prms, expand_prms = {}, {}
-    for popt in popts:
-        popt = popt.split("=")
-        if len(popt) == 1:
-            # Implicit expand in full parameter space -- not yet completely
-            # implemented
-            intr_prms[popt[0]] = None
-        elif len(popt) == 2:
-            popt[1] = popt[1].split(",")
-            if len(popt[1]) == 1:
-                # Fix intrinsic point
-                intr_prms[popt[0]] = float(popt[1][0])
-            else:
-                expand_prms[popt[0]] = tuple(map(float, popt[1]))
-    return intr_prms, expand_prms
-
 argp = ArgumentParser()
 
 argp.add_argument("-d", "--distance-coordinates", default="tau0_tau3", help="Coordinate system in which to calculate 'closeness'. Default is tau0_tau3.")
@@ -241,9 +212,9 @@ if opts.use_overlap is not None:
 # could incur an overlap calculation, or suffer from the effects of being close
 # only in Euclidean terms
 
-intr_prms, expand_prms = parse_param(opts.intrinsic_param)
-pin_prms, _ = parse_param(opts.pin_param)
-intr_pt = numpy.array([intr_prms[k] for k in intr_prms])
+intr_prms, expand_prms = common_cl.parse_param(opts.intrinsic_param)
+pin_prms, _ = common_cl.parse_param(opts.pin_param)
+intr_pt = numpy.array([intr_prms[k] for k in sorted(intr_prms)])
 # This keeps the list of parameters consistent across runs
 intr_prms = sorted(intr_prms.keys())
 
@@ -261,17 +232,26 @@ tmplt_bank = lsctables.SnglInspiralTable.get_table(xmldoc)
 # Step 2: Set up metric space
 #
 
-# NOTE: We use the template bank here because the overlap results might not have
-# all the intrinsic information stored (e.g.: no spins, even though the bank is
-# aligned-spin).
-# FIXME: This is an oversight in the overlap calculator that should be
-# rectified.
-pts = numpy.array([tuple(getattr(t, a) for a in intr_prms) for t in tmplt_bank])
+if ovrlp.shape[1] != len(tmplt_bank):
+    pts = numpy.array([odata[a] for a in intr_prms]).T
+else:
+    # NOTE: We use the template bank here because the overlap results might not
+    # have all the intrinsic information stored (e.g.: no spins, even though the
+    # bank is aligned-spin).
+    # FIXME: This is an oversight in the overlap calculator which was rectified
+    # but this remains for legacy banks
+    pts = numpy.array([tuple(getattr(t, a) for a in intr_prms) for t in tmplt_bank])
+
 pts = amrlib.apply_transform(pts, intr_prms, opts.distance_coordinates)
 
 # FIXME: Can probably be moved to point index identification function -- it's
 # not used again
-tree = BallTree(pts)
+# The slicing here is a slight hack to work around uberbank overlaps where the
+# overlap matrix is non square. This can be slightly dangerous because it
+# assumes the first N points are from the bank in question. That's okay for now
+# but we're getting increasingly complex in how we do construction, so we should
+# be more sophisticated by matching template IDs instead.
+tree = BallTree(pts[:,:ovrlp.shape[0]))
 
 #
 # Step 3: Get the row of the overlap matrix to work with
@@ -372,7 +352,8 @@ else:
 # FIXME: We just need to be consistent from the beginning
 reindex = numpy.array([list(region_labels).index(l) for l in intr_prms])
 intr_prms = list(region_labels)
-res_pts = res_pts[:,reindex]
+if opts.refine or opts.prerefine:
+    res_pts = res_pts[:,reindex]
 
 extent_str = " ".join("(%f, %f)" % bnd for bnd in map(tuple, init_region._bounds))
 center_str = " ".join(map(str, init_region._center))
