@@ -208,8 +208,6 @@ static void q2masses(double mc, double q, double *m1, double *m2)
 void LALInferenceROQWrapperForXLALSimInspiralChooseFDWaveformSequence(LALInferenceModel *model){
 /*************************************************************************************************************************/
   Approximant approximant = (Approximant) 0;
-  INT4 order=-1;
-  INT4 amporder;
 
   int ret=0;
   INT4 errnum=0;
@@ -229,7 +227,7 @@ void LALInferenceROQWrapperForXLALSimInspiralChooseFDWaveformSequence(LALInferen
   }
 
   if (LALInferenceCheckVariable(model->params, "LAL_PNORDER"))
-    order = *(INT4*) LALInferenceGetVariable(model->params, "LAL_PNORDER");
+    XLALSimInspiralWaveformParamsInsertPNPhaseOrder(model->LALpars, *(INT4 *) LALInferenceGetVariable(model->params, "LAL_PNORDER"));
   else {
     XLALPrintError(" ERROR in templateLALGenerateInspiral(): (INT4) \"LAL_PNORDER\" parameter not provided!\n");
     XLAL_ERROR_VOID(XLAL_EDATA);
@@ -240,9 +238,11 @@ void LALInferenceROQWrapperForXLALSimInspiralChooseFDWaveformSequence(LALInferen
    *     1) The default behavior of the code won't change unexpectedly due to changes in LALSimulation.
    *     2) We need to know the amplitude order in order to set the starting frequency of the waveform properly. */
   if (LALInferenceCheckVariable(model->params, "LAL_AMPORDER"))
-    amporder = *(INT4*) LALInferenceGetVariable(model->params, "LAL_AMPORDER");
+    XLALSimInspiralWaveformParamsInsertPNAmplitudeOrder(model->LALpars, *(INT4 *) LALInferenceGetVariable(model->params, "LAL_AMPORDER"));
   else
-    amporder = -1;
+    if (!XLALSimInspiralWaveformParamsPNAmplitudeOrderIsDefault(model->LALpars))
+      XLALSimInspiralWaveformParamsInsertPNAmplitudeOrder(model->LALpars,-1);
+
   REAL8 f_ref = 100.0;
   if (LALInferenceCheckVariable(model->params, "f_ref")) f_ref = *(REAL8 *)LALInferenceGetVariable(model->params, "f_ref");
 
@@ -325,7 +325,7 @@ void LALInferenceROQWrapperForXLALSimInspiralChooseFDWaveformSequence(LALInferen
       /* The transformation function doesn't know fLow, so f_ref==0 isn't interpretted as a request to use the starting frequency for reference. */
       XLAL_TRY(ret=XLALSimInspiralTransformPrecessingNewInitialConditions(
                     &inclination, &spin1x, &spin1y, &spin1z, &spin2x, &spin2y, &spin2z,
-                    thetaJN, phiJL, tilt1, tilt2, phi12, a_spin1, a_spin2, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, fTemp), errnum);
+                    thetaJN, phiJL, tilt1, tilt2, phi12, a_spin1, a_spin2, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, fTemp, phi0), errnum);
       if (ret == XLAL_FAILURE)
       {
         XLALPrintError(" ERROR in XLALSimInspiralTransformPrecessingNewInitialConditions(): error converting angles. errnum=%d\n",errnum );
@@ -334,30 +334,35 @@ void LALInferenceROQWrapperForXLALSimInspiralChooseFDWaveformSequence(LALInferen
   }
 
   /* ==== TIDAL PARAMETERS ==== */
-  REAL8 lambda1 = 0.;
-  if(LALInferenceCheckVariable(model->params, "lambda1")) lambda1 = *(REAL8*) LALInferenceGetVariable(model->params, "lambda1");
-  REAL8 lambda2 = 0.;
-  if(LALInferenceCheckVariable(model->params, "lambda2")) lambda2 = *(REAL8*) LALInferenceGetVariable(model->params, "lambda2");
+  if(LALInferenceCheckVariable(model->params, "lambda1"))
+    XLALSimInspiralWaveformParamsInsertTidalLambda1(model->LALpars, *(REAL8*) LALInferenceGetVariable(model->params, "lambda1"));
+  if(LALInferenceCheckVariable(model->params, "lambda2"))
+    XLALSimInspiralWaveformParamsInsertTidalLambda2(model->LALpars, *(REAL8*) LALInferenceGetVariable(model->params, "lambda2"));
   REAL8 lambdaT = 0.;
   REAL8 dLambdaT = 0.;
   REAL8 sym_mass_ratio_eta = 0.;
+
   if(LALInferenceCheckVariable(model->params, "lambdaT")&&LALInferenceCheckVariable(model->params, "dLambdaT")){
+    REAL8 lambda1=0.;
+    REAL8 lambda2=0.;
     lambdaT = *(REAL8*) LALInferenceGetVariable(model->params, "lambdaT");
     dLambdaT = *(REAL8*) LALInferenceGetVariable(model->params, "dLambdaT");
     sym_mass_ratio_eta = m1*m2/((m1+m2)*(m1+m2));
     LALInferenceLambdaTsEta2Lambdas(lambdaT,dLambdaT,sym_mass_ratio_eta,&lambda1,&lambda2);
+    XLALSimInspiralWaveformParamsInsertTidalLambda1(model->LALpars,lambda1);
+    XLALSimInspiralWaveformParamsInsertTidalLambda2(model->LALpars,lambda2);
   }
 
 
   /* Only use GR templates */
-  LALSimInspiralTestGRParam *nonGRparams = NULL;
-
   /* Fill in the extra parameters for testing GR, if necessary */
   for (UINT4 k=0; k<N_extra_params; k++)
   {
     if(LALInferenceCheckVariable(model->params,list_extra_parameters[k]))
     {
-      XLALSimInspiralAddTestGRParam(&nonGRparams,list_extra_parameters[k],*(REAL8 *)LALInferenceGetVariable(model->params,list_extra_parameters[k]));
+      XLALDictInsert(model->LALpars, list_extra_parameters[k], (void *)LALInferenceGetVariable(model->params,list_extra_parameters[k]), sizeof(double), LAL_D_TYPE_CODE);
+
+      //XLALSimInspiralAddTestGRParam(&nonGRparams,list_extra_parameters[k],*(REAL8 *)LALInferenceGetVariable(model->params,list_extra_parameters[k]));
     }
   }
   /* Fill in PPE params if they are available */
@@ -369,22 +374,18 @@ void LALInferenceROQWrapperForXLALSimInspiralChooseFDWaveformSequence(LALInferen
     {
       sprintf(PPEparam, "%s%d",PPEnames[idx],ppeidx);
       if(LALInferenceCheckVariable(model->params,PPEparam))
-        XLALSimInspiralAddTestGRParam(&nonGRparams,PPEparam,LALInferenceGetREAL8Variable(model->params,PPEparam));
+	XLALDictInsert(model->LALpars, PPEparam, (void *)LALInferenceGetVariable(model->params,PPEparam), sizeof(double), LAL_D_TYPE_CODE);
       else
         break;
     }
   }
 
   /* ==== Call the waveform generator ==== */
-  XLAL_TRY(ret=XLALSimInspiralChooseFDWaveformSequence (&(model->roq->hptildeLinear), &(model->roq->hctildeLinear), phi0, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI,
-                spin1x, spin1y, spin1z, spin2x, spin2y, spin2z, f_ref, distance, inclination, lambda1, lambda2,
-                model->waveFlags, nonGRparams, amporder, order, approximant, (model->roq->frequencyNodesLinear)), errnum);
+    XLAL_TRY(ret=XLALSimInspiralChooseFDWaveformSequence (&(model->roq->hptildeLinear), &(model->roq->hctildeLinear), phi0, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI,
+                spin1x, spin1y, spin1z, spin2x, spin2y, spin2z, f_ref, distance, inclination, model->LALpars, approximant, (model->roq->frequencyNodesLinear)), errnum);
 
-  XLAL_TRY(ret=XLALSimInspiralChooseFDWaveformSequence (&(model->roq->hptildeQuadratic), &(model->roq->hctildeQuadratic), phi0, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI,
-                  spin1x, spin1y, spin1z, spin2x, spin2y, spin2z, f_ref, distance, inclination, lambda1, lambda2,
-                  model->waveFlags, nonGRparams, amporder, order, approximant, (model->roq->frequencyNodesQuadratic)), errnum);
-    /* Destroy the nonGr params */
-    XLALSimInspiralDestroyTestGRParam(nonGRparams);
+    XLAL_TRY(ret=XLALSimInspiralChooseFDWaveformSequence (&(model->roq->hptildeQuadratic), &(model->roq->hctildeQuadratic), phi0, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI,
+							spin1x, spin1y, spin1z, spin2x, spin2y, spin2z, f_ref, distance, inclination, model->LALpars, approximant, (model->roq->frequencyNodesQuadratic)), errnum);
 
     REAL8 instant = model->freqhPlus->epoch.gpsSeconds + 1e-9*model->freqhPlus->epoch.gpsNanoSeconds;
     LALInferenceSetVariable(model->params, "time", &instant);
@@ -608,8 +609,6 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveform(LALInferenceModel *model)
 {
 
   Approximant approximant = (Approximant) 0;
-  INT4 order=-1;
-  INT4 amporder;
 
   static int sizeWarning = 0;
   int ret=0;
@@ -635,7 +634,7 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveform(LALInferenceModel *model)
   }
 
   if (LALInferenceCheckVariable(model->params, "LAL_PNORDER"))
-    order = *(INT4*) LALInferenceGetVariable(model->params, "LAL_PNORDER");
+    XLALSimInspiralWaveformParamsInsertPNPhaseOrder(model->LALpars, *(INT4*) LALInferenceGetVariable(model->params, "LAL_PNORDER"));
   else {
     XLALPrintError(" ERROR in templateLALGenerateInspiral(): (INT4) \"LAL_PNORDER\" parameter not provided!\n");
     XLAL_ERROR_VOID(XLAL_EDATA);
@@ -646,9 +645,10 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveform(LALInferenceModel *model)
    *     1) The default behavior of the code won't change unexpectedly due to changes in LALSimulation.
    *     2) We need to know the amplitude order in order to set the starting frequency of the waveform properly. */
   if (LALInferenceCheckVariable(model->params, "LAL_AMPORDER"))
-    amporder = *(INT4*) LALInferenceGetVariable(model->params, "LAL_AMPORDER");
+    XLALSimInspiralWaveformParamsInsertPNAmplitudeOrder(model->LALpars,*(INT4*) LALInferenceGetVariable(model->params, "LAL_AMPORDER"));
   else
-    amporder = -1;
+    if (!XLALSimInspiralWaveformParamsPNAmplitudeOrderIsDefault(model->LALpars))
+      XLALSimInspiralWaveformParamsInsertPNAmplitudeOrder(model->LALpars,-1);
 
   REAL8 f_ref = 100.0;
   if (LALInferenceCheckVariable(model->params, "f_ref")) f_ref = *(REAL8 *)LALInferenceGetVariable(model->params, "f_ref");
@@ -690,7 +690,7 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveform(LALInferenceModel *model)
   else
     f_low = model->fLow;
 
-  f_start = XLALSimInspiralfLow2fStart(f_low, amporder, approximant);
+  f_start = XLALSimInspiralfLow2fStart(f_low, XLALSimInspiralWaveformParamsLookupPNAmplitudeOrder(model->LALpars), approximant);
   f_max = 0.0; /* for freq domain waveforms this will stop at ISCO. Previously found using model->fHigh causes NaNs in waveform (see redmine issue #750)*/
 
   /* ==== SPINS ==== */
@@ -746,7 +746,7 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveform(LALInferenceModel *model)
 
       XLAL_TRY(ret=XLALSimInspiralTransformPrecessingNewInitialConditions(
                     &inclination, &spin1x, &spin1y, &spin1z, &spin2x, &spin2y, &spin2z,
-                    thetaJN, phiJL, tilt1, tilt2, phi12, a_spin1, a_spin2, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, fTemp), errnum);
+                    thetaJN, phiJL, tilt1, tilt2, phi12, a_spin1, a_spin2, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, fTemp, phi0), errnum);
       if (ret == XLAL_FAILURE)
       {
         XLALPrintError(" ERROR in XLALSimInspiralTransformPrecessingNewInitialConditions(): error converting angles. errnum=%d: %s\n",errnum, XLALErrorString(errnum) );
@@ -756,30 +756,32 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveform(LALInferenceModel *model)
 
 
   /* ==== TIDAL PARAMETERS ==== */
-  REAL8 lambda1 = 0.;
-  if(LALInferenceCheckVariable(model->params, "lambda1")) lambda1 = *(REAL8*) LALInferenceGetVariable(model->params, "lambda1");
-  REAL8 lambda2 = 0.;
-  if(LALInferenceCheckVariable(model->params, "lambda2")) lambda2 = *(REAL8*) LALInferenceGetVariable(model->params, "lambda2");
+  if(LALInferenceCheckVariable(model->params, "lambda1"))
+    XLALSimInspiralWaveformParamsInsertTidalLambda1(model->LALpars, *(REAL8*) LALInferenceGetVariable(model->params, "lambda1"));
+  if(LALInferenceCheckVariable(model->params, "lambda2"))
+    XLALSimInspiralWaveformParamsInsertTidalLambda2(model->LALpars, *(REAL8*) LALInferenceGetVariable(model->params, "lambda2"));
   REAL8 lambdaT = 0.;
   REAL8 dLambdaT = 0.;
   REAL8 sym_mass_ratio_eta = 0.;
   if(LALInferenceCheckVariable(model->params, "lambdaT")&&LALInferenceCheckVariable(model->params, "dLambdaT")){
+    REAL8 lambda1=0.;
+    REAL8 lambda2=0.;
     lambdaT = *(REAL8*) LALInferenceGetVariable(model->params, "lambdaT");
     dLambdaT = *(REAL8*) LALInferenceGetVariable(model->params, "dLambdaT");
     sym_mass_ratio_eta = m1*m2/((m1+m2)*(m1+m2));
     LALInferenceLambdaTsEta2Lambdas(lambdaT,dLambdaT,sym_mass_ratio_eta,&lambda1,&lambda2);
+    XLALSimInspiralWaveformParamsInsertTidalLambda1(model->LALpars, lambda1);
+    XLALSimInspiralWaveformParamsInsertTidalLambda2(model->LALpars, lambda2);
   }
 
 
   /* Only use GR templates */
-  LALSimInspiralTestGRParam *nonGRparams = NULL;
-
   /* Fill in the extra parameters for testing GR, if necessary */
   for (UINT4 k=0; k<N_extra_params; k++)
   {
       if(LALInferenceCheckVariable(model->params,list_extra_parameters[k]))
       {
-          XLALSimInspiralAddTestGRParam(&nonGRparams,list_extra_parameters[k],*(REAL8 *)LALInferenceGetVariable(model->params,list_extra_parameters[k]));
+	XLALDictInsert(model->LALpars, list_extra_parameters[k], (void *)LALInferenceGetVariable(model->params,list_extra_parameters[k]), sizeof(double), LAL_D_TYPE_CODE);
       }
   }
   /* Fill in PPE params if they are available */
@@ -791,7 +793,7 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveform(LALInferenceModel *model)
     {
       sprintf(PPEparam, "%s%d",PPEnames[idx],ppeidx);
       if(LALInferenceCheckVariable(model->params,PPEparam))
-        XLALSimInspiralAddTestGRParam(&nonGRparams,PPEparam,LALInferenceGetREAL8Variable(model->params,PPEparam));
+	XLALDictInsert(model->LALpars, PPEparam, (void *)LALInferenceGetVariable(model->params,PPEparam), sizeof(double), LAL_D_TYPE_CODE);
       else
         break;
     }
@@ -802,12 +804,10 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveform(LALInferenceModel *model)
   /* ==== Call the waveform generator ==== */
   if(model->domain == LAL_SIM_DOMAIN_FREQUENCY) {
     deltaF = model->deltaF;
-
-	XLAL_TRY(ret=XLALSimInspiralChooseFDWaveformFromCache(&hptilde, &hctilde, phi0,
+    XLAL_TRY(ret=XLALSimInspiralChooseFDWaveformFromCache(&hptilde, &hctilde, phi0,
             deltaF, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, spin1x, spin1y, spin1z,
-            spin2x, spin2y, spin2z, f_start, f_max, f_ref, distance, inclination,lambda1, lambda2, model->waveFlags, nonGRparams, amporder, order,
-            approximant,model->waveformCache, NULL), errnum);
-
+            spin2x, spin2y, spin2z, f_start, f_max, f_ref, distance, inclination, model->LALpars,
+							      approximant,model->waveformCache, NULL), errnum);
 
     /* if the waveform failed to generate, fill the buffer with zeros
      * so that the previous waveform is not left there
@@ -817,7 +817,6 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveform(LALInferenceModel *model)
       memset(model->freqhCross->data->data,0,sizeof(model->freqhCross->data->data[0])*model->freqhCross->data->length);
       if ( hptilde ) XLALDestroyCOMPLEX16FrequencySeries(hptilde);
       if ( hctilde ) XLALDestroyCOMPLEX16FrequencySeries(hctilde);
-      XLALSimInspiralDestroyTestGRParam(nonGRparams);
       errnum&=~XLAL_EFUNC; /* Mask out the internal function failure bit */
       switch(errnum)
       {
@@ -829,12 +828,10 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveform(LALInferenceModel *model)
           XLALSetErrno(errnum);
           XLAL_ERROR_VOID(errnum,"%s: Template generation failed in XLALSimInspiralChooseFDWaveformFromCache:\n\
 XLALSimInspiralChooseFDWaveformFromCache(&hptilde, &hctilde, \
-%g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, \
-model->waveFlags(%d,%d,%d,%d,numreldata),nonGRparams,%d,%d,%d,model->waveformCache)\n",__func__,
-            phi0, deltaF, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, spin1x, spin1y, spin1z, spin2x, spin2y, spin2z,
-            f_start, f_max, f_ref, distance, inclination, lambda1, lambda2, (int) XLALSimInspiralGetSpinOrder(model->waveFlags),
-            (int) XLALSimInspiralGetTidalOrder(model->waveFlags),(int) XLALSimInspiralGetFrameAxis(model->waveFlags),
-            (int) XLALSimInspiralGetModesChoice(model->waveFlags),amporder, order, approximant);
+%g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, \
+,model->LALpars,model->waveformCache)\n",__func__,
+			  phi0, deltaF, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, spin1x, spin1y, spin1z, spin2x, spin2y, spin2z,
+			  f_start, f_max, f_ref, distance, inclination);
       }
     }
 
@@ -868,8 +865,7 @@ model->waveFlags(%d,%d,%d,%d,numreldata),nonGRparams,%d,%d,%d,model->waveformCac
     XLAL_TRY(ret=XLALSimInspiralChooseTDWaveformFromCache(&hplus, &hcross, phi0, deltaT,
             m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, spin1x, spin1y, spin1z,
             spin2x, spin2y, spin2z, f_start, f_ref, distance,
-            inclination, lambda1, lambda2, model->waveFlags, nonGRparams,
-            amporder, order, approximant,model->waveformCache), errnum);
+	    inclination, model->LALpars, approximant,model->waveformCache), errnum);
     /* if the waveform failed to generate, fill the buffer with zeros
      * so that the previous waveform is not left there
      */
@@ -880,7 +876,6 @@ model->waveFlags(%d,%d,%d,%d,numreldata),nonGRparams,%d,%d,%d,model->waveformCac
       if ( hctilde ) XLALDestroyCOMPLEX16FrequencySeries(hctilde);
       if ( hplus) XLALDestroyREAL8TimeSeries(hplus);
       if ( hcross ) XLALDestroyREAL8TimeSeries(hcross);
-      XLALSimInspiralDestroyTestGRParam(nonGRparams);
       errnum&=~XLAL_EFUNC; /* Mask out the internal function failure bit */
       switch(errnum)
       {
@@ -892,15 +887,14 @@ model->waveFlags(%d,%d,%d,%d,numreldata),nonGRparams,%d,%d,%d,model->waveformCac
           XLALSetErrno(errnum);
           XLAL_ERROR_VOID(errnum,"%s: Template generation failed in XLALSimInspiralChooseTDWaveformFromCache\n\
 XLALSimInspiralChooseTDWaveformFromCache(&hplus, &hcross, \
-%g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, \
-model->waveFlags(%d,%d,%d,%d,numreldata),nonGRparams,%d,%d,%d,model->waveformCache)\n",__func__,
+%g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, \
+model->LALpars,model->waveformCache)\n",__func__,
             phi0, deltaT, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, spin1x, spin1y, spin1z, spin2x, spin2y, spin2z,
-            f_start, f_ref, distance, inclination, lambda1, lambda2, (int) XLALSimInspiralGetSpinOrder(model->waveFlags),
-            (int) XLALSimInspiralGetTidalOrder(model->waveFlags),(int) XLALSimInspiralGetFrameAxis(model->waveFlags),
-            (int) XLALSimInspiralGetModesChoice(model->waveFlags),amporder, order, approximant);
+            f_start, f_ref, distance, inclination);
       }
     }
 
+
     /* The following complicated mess is a result of the following considerations:
 
        1) The discrete time samples of the template and the timeModel
@@ -1087,8 +1081,6 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
 /*************************************************************************************************************************/
 {
     Approximant approximant = (Approximant) 0;
-    INT4 order=-1;
-    INT4 amporder;
 
     static int sizeWarning = 0;
     int ret=0;
@@ -1112,8 +1104,10 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
         XLAL_ERROR_VOID(XLAL_EDATA);
     }
 
-    if (LALInferenceCheckVariable(model->params, "LAL_PNORDER"))
-        order = *(INT4*) LALInferenceGetVariable(model->params, "LAL_PNORDER");
+    if (LALInferenceCheckVariable(model->params, "LAL_PNORDER")) {
+      if (XLALSimInspiralWaveformParamsLookupPNPhaseOrder(model->LALpars)!=(*(INT4*) LALInferenceGetVariable(model->params, "LAL_PNORDER")))
+	XLALSimInspiralWaveformParamsInsertPNPhaseOrder(model->LALpars, *(INT4*) LALInferenceGetVariable(model->params, "LAL_PNORDER"));
+    }
     else {
         XLALPrintError(" ERROR in templateLALGenerateInspiral(): (INT4) \"LAL_PNORDER\" parameter not provided!\n");
         XLAL_ERROR_VOID(XLAL_EDATA);
@@ -1124,10 +1118,13 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
      *     1) The default behavior of the code won't change unexpectedly due to changes in LALSimulation.
      *     2) We need to know the amplitude order in order to set the starting frequency of the waveform properly. */
 
-    if (LALInferenceCheckVariable(model->params, "LAL_AMPORDER"))
-        amporder = *(INT4*) LALInferenceGetVariable(model->params, "LAL_AMPORDER");
+    if (LALInferenceCheckVariable(model->params, "LAL_AMPORDER")) {
+      if ( XLALSimInspiralWaveformParamsLookupPNAmplitudeOrder(model->LALpars)!= *(INT4*) LALInferenceGetVariable(model->params, "LAL_AMPORDER"))
+	XLALSimInspiralWaveformParamsInsertPNAmplitudeOrder(model->LALpars, *(INT4*) LALInferenceGetVariable(model->params, "LAL_AMPORDER"));
+    }
     else
-        amporder = -1;
+      if (!XLALSimInspiralWaveformParamsPNAmplitudeOrderIsDefault(model->LALpars))
+	XLALSimInspiralWaveformParamsInsertPNAmplitudeOrder(model->LALpars,-1);
 
     REAL8 f_ref = 100.0;
     if (LALInferenceCheckVariable(model->params, "f_ref")) f_ref = *(REAL8 *)LALInferenceGetVariable(model->params, "f_ref");
@@ -1169,7 +1166,7 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
     else
         f_low = model->fLow;
 
-    f_start = XLALSimInspiralfLow2fStart(f_low, amporder, approximant);
+    f_start = XLALSimInspiralfLow2fStart(f_low, XLALSimInspiralWaveformParamsLookupPNAmplitudeOrder(model->LALpars), approximant);
     f_max = 0.0; /* for freq domain waveforms this will stop at ISCO. Previously found using model->fHigh causes NaNs in waveform (see redmine issue #750)*/
 
     /* ==== SPINS ==== */
@@ -1225,7 +1222,7 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
 
         XLAL_TRY(ret=XLALSimInspiralTransformPrecessingNewInitialConditions(
                                                                             &inclination, &spin1x, &spin1y, &spin1z, &spin2x, &spin2y, &spin2z,
-                                                                            thetaJN, phiJL, tilt1, tilt2, phi12, a_spin1, a_spin2, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, fTemp), errnum);
+                                                                            thetaJN, phiJL, tilt1, tilt2, phi12, a_spin1, a_spin2, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, fTemp, phi0), errnum);
         if (ret == XLAL_FAILURE)
         {
             XLALPrintError(" ERROR in XLALSimInspiralTransformPrecessingNewInitialConditions(): error converting angles. errnum=%d: %s\n",errnum, XLALErrorString(errnum) );
@@ -1234,28 +1231,31 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
     }
 
     /* ==== TIDAL PARAMETERS ==== */
-    REAL8 lambda1 = 0.;
-    if(LALInferenceCheckVariable(model->params, "lambda1")) lambda1 = *(REAL8*) LALInferenceGetVariable(model->params, "lambda1");
-    REAL8 lambda2 = 0.;
-    if(LALInferenceCheckVariable(model->params, "lambda2")) lambda2 = *(REAL8*) LALInferenceGetVariable(model->params, "lambda2");
+    if(LALInferenceCheckVariable(model->params, "lambda1"))
+      XLALSimInspiralWaveformParamsInsertTidalLambda1(model->LALpars, *(REAL8*) LALInferenceGetVariable(model->params, "lambda1"));
+    if(LALInferenceCheckVariable(model->params, "lambda2"))
+      XLALSimInspiralWaveformParamsInsertTidalLambda2(model->LALpars, *(REAL8*) LALInferenceGetVariable(model->params, "lambda2"));
     REAL8 lambdaT = 0.;
     REAL8 dLambdaT = 0.;
     REAL8 sym_mass_ratio_eta = 0.;
     if(LALInferenceCheckVariable(model->params, "lambdaT")&&LALInferenceCheckVariable(model->params, "dLambdaT")){
+        REAL8 lambda1=0.;
+        REAL8 lambda2=0.;
         lambdaT = *(REAL8*) LALInferenceGetVariable(model->params, "lambdaT");
         dLambdaT = *(REAL8*) LALInferenceGetVariable(model->params, "dLambdaT");
         sym_mass_ratio_eta = m1*m2/((m1+m2)*(m1+m2));
         LALInferenceLambdaTsEta2Lambdas(lambdaT,dLambdaT,sym_mass_ratio_eta,&lambda1,&lambda2);
+        XLALSimInspiralWaveformParamsInsertTidalLambda1(model->LALpars, lambda1);
+        XLALSimInspiralWaveformParamsInsertTidalLambda2(model->LALpars, lambda2);
     }
 
     /* Only use GR templates */
-    LALSimInspiralTestGRParam *nonGRparams = NULL;
     /* Fill in the extra parameters for testing GR, if necessary */
     for (UINT4 k=0; k<N_extra_params; k++)
     {
         if(LALInferenceCheckVariable(model->params,list_extra_parameters[k]))
         {
-            XLALSimInspiralAddTestGRParam(&nonGRparams,list_extra_parameters[k],*(REAL8 *)LALInferenceGetVariable(model->params,list_extra_parameters[k]));
+	  XLALDictInsert(model->LALpars, list_extra_parameters[k], (void *)LALInferenceGetVariable(model->params,list_extra_parameters[k]), sizeof(double), LAL_D_TYPE_CODE);
         }
     }
     /* Fill in PPE params if they are available */
@@ -1267,7 +1267,7 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
         {
             sprintf(PPEparam, "%s%d",PPEnames[idx],ppeidx);
             if(LALInferenceCheckVariable(model->params,PPEparam))
-                XLALSimInspiralAddTestGRParam(&nonGRparams,PPEparam,LALInferenceGetREAL8Variable(model->params,PPEparam));
+	      XLALDictInsert(model->LALpars, PPEparam, (void *)LALInferenceGetVariable(model->params,PPEparam), sizeof(double), LAL_D_TYPE_CODE);
             else
                 break;
         }
@@ -1285,7 +1285,7 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
 
         XLAL_TRY(ret=XLALSimInspiralChooseFDWaveformFromCache(&hptilde, &hctilde, phi0,
                                                               0.0, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, spin1x, spin1y, spin1z,
-                                                              spin2x, spin2y, spin2z, f_start, f_max, f_ref, distance, inclination,lambda1, lambda2, model->waveFlags, nonGRparams, amporder, order,
+                                                              spin2x, spin2y, spin2z, f_start, f_max, f_ref, distance, inclination, model->LALpars,
                                                               approximant,model->waveformCache, frequencies), errnum);
 
         /* if the waveform failed to generate, fill the buffer with zeros
@@ -1296,7 +1296,6 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
             memset(model->freqhCross->data->data,0,sizeof(model->freqhCross->data->data[0])*model->freqhCross->data->length);
             if ( hptilde ) XLALDestroyCOMPLEX16FrequencySeries(hptilde);
             if ( hctilde ) XLALDestroyCOMPLEX16FrequencySeries(hctilde);
-            XLALSimInspiralDestroyTestGRParam(nonGRparams);
             errnum&=~XLAL_EFUNC; /* Mask out the internal function failure bit */
             switch(errnum)
             {
@@ -1308,12 +1307,10 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
                     XLALSetErrno(errnum);
                     XLAL_ERROR_VOID(errnum,"%s: Template generation failed in XLALSimInspiralChooseFDWaveformFromCache:\n\
 XLALSimInspiralChooseFDWaveformFromCache(&hptilde, &hctilde, \
-%g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, \
-model->waveFlags(%d,%d,%d,%d,numreldata),nonGRparams,%d,%d,%d,model->waveformCache)\n",__func__,
+%g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, \
+model->LALpars,%d,model->waveformCache)\n",__func__,
 		      phi0, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, spin1x, spin1y, spin1z, spin2x, spin2y, spin2z,
-		      f_start, f_max, f_ref, distance, inclination, lambda1, lambda2, (int) XLALSimInspiralGetSpinOrder(model->waveFlags),
-		      (int) XLALSimInspiralGetTidalOrder(model->waveFlags),(int) XLALSimInspiralGetFrameAxis(model->waveFlags),
-		      (int) XLALSimInspiralGetModesChoice(model->waveFlags),amporder, order, approximant);
+				    f_start, f_max, f_ref, distance, inclination, approximant);
             }
         }
 
@@ -1330,9 +1327,6 @@ model->waveFlags(%d,%d,%d,%d,numreldata),nonGRparams,%d,%d,%d,model->waveformCac
         InterpolateWaveform(frequencies, hptilde, model->freqhPlus);
         InterpolateWaveform(frequencies, hctilde, model->freqhCross);
 
-        /* Destroy the nonGr params */
-        XLALSimInspiralDestroyTestGRParam(nonGRparams);
-
         REAL8 instant = model->freqhPlus->epoch.gpsSeconds + 1e-9*model->freqhPlus->epoch.gpsNanoSeconds;
         LALInferenceSetVariable(model->params, "time", &instant);
 
@@ -1342,8 +1336,8 @@ model->waveFlags(%d,%d,%d,%d,numreldata),nonGRparams,%d,%d,%d,model->waveformCac
         XLAL_TRY(ret=XLALSimInspiralChooseTDWaveformFromCache(&hplus, &hcross, phi0, deltaT,
                                                               m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, spin1x, spin1y, spin1z,
                                                               spin2x, spin2y, spin2z, f_start, f_ref, distance,
-                                                              inclination, lambda1, lambda2, model->waveFlags, nonGRparams,
-                                                              amporder, order, approximant,model->waveformCache), errnum);
+                                                              inclination, model->LALpars,
+                                                              approximant,model->waveformCache), errnum);
         /* if the waveform failed to generate, fill the buffer with zeros
          * so that the previous waveform is not left there
          */
@@ -1352,7 +1346,6 @@ model->waveFlags(%d,%d,%d,%d,numreldata),nonGRparams,%d,%d,%d,model->waveformCac
             memset(model->freqhCross->data->data,0,sizeof(model->freqhCross->data->data[0])*model->freqhCross->data->length);
             if ( hptilde ) XLALDestroyCOMPLEX16FrequencySeries(hptilde);
             if ( hctilde ) XLALDestroyCOMPLEX16FrequencySeries(hctilde);
-            XLALSimInspiralDestroyTestGRParam(nonGRparams);
             errnum&=~XLAL_EFUNC; /* Mask out the internal function failure bit */
             switch(errnum)
             {
