@@ -1,5 +1,6 @@
 /*
- * LALInferenceVCSInfo.h - LALInference VCS Information Header
+ * Copyright (C) 2014, 2016 Karl Wette
+ * Copyright (C) 2009-2013 Adam Mercer
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,9 +16,10 @@
  * along with with program; see the file COPYING. If not, write to the
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  * MA 02111-1307 USA
- *
- * Copyright (C) 2009-2013 Adam Mercer
- * Copyright (C) 2014 Karl Wette
+ */
+
+/*
+ * LALInferenceVCSInfo.h - LALInference VCS Information Header
  */
 
 /** \cond DONT_DOXYGEN */
@@ -33,37 +35,19 @@
 extern "C" {
 #endif
 
-/* VCS information defines */
-#define LALINFERENCE_NAME "LALInference"
-#define LALINFERENCE_VCS_ID "f3fc41aed5022cf091038952048fe78f52e82cdf"
-#define LALINFERENCE_VCS_DATE "2016-10-10 17:31:35 +0000"
+/* VCS information */
+#define LALINFERENCE_VCS_ID "cbb23a10b4a59e71d483d5d1eefbe78dfbee4b5f"
+#define LALINFERENCE_VCS_DATE "2017-07-27 17:30:32 +0000"
 #define LALINFERENCE_VCS_BRANCH "None"
-#define LALINFERENCE_VCS_TAG "lalinference-v1.8.0"
+#define LALINFERENCE_VCS_TAG "lalinference-v1.9.4"
 #define LALINFERENCE_VCS_AUTHOR "Adam Mercer <adam.mercer@ligo.org>"
 #define LALINFERENCE_VCS_COMMITTER "Adam Mercer <adam.mercer@ligo.org>"
+#define LALINFERENCE_VCS_CLEAN "CLEAN"
 #define LALINFERENCE_VCS_STATUS "CLEAN: All modifications committed"
 
-/* Identable VCS information defines */
-#define LALINFERENCE_VCS_IDENT_ID "$" "LALInferenceId: f3fc41aed5022cf091038952048fe78f52e82cdf " "$"
-#define LALINFERENCE_VCS_IDENT_DATE "$" "LALInferenceDate: 2016-10-10 17:31:35 +0000 " "$"
-#define LALINFERENCE_VCS_IDENT_BRANCH "$" "LALInferenceBranch: None " "$"
-#define LALINFERENCE_VCS_IDENT_TAG "$" "LALInferenceTag: lalinference-v1.8.0 " "$"
-#define LALINFERENCE_VCS_IDENT_AUTHOR "$" "LALInferenceAuthor: Adam Mercer <adam.mercer@ligo.org> " "$"
-#define LALINFERENCE_VCS_IDENT_COMMITTER "$" "LALInferenceCommitter: Adam Mercer <adam.mercer@ligo.org> " "$"
-#define LALINFERENCE_VCS_IDENT_STATUS "$" "LALInferenceStatus: CLEAN: All modifications committed " "$"
-
-/* Header VCS information structure */
-static const LALVCSInfo lalInferenceVCSInfoHeader = { \
-  LALINFERENCE_NAME, \
-  LALINFERENCE_VERSION, \
-  LALINFERENCE_VCS_ID, \
-  LALINFERENCE_VCS_DATE, \
-  LALINFERENCE_VCS_BRANCH, \
-  LALINFERENCE_VCS_TAG, \
-  LALINFERENCE_VCS_AUTHOR, \
-  LALINFERENCE_VCS_COMMITTER, \
-  LALINFERENCE_VCS_STATUS \
-};
+/* VCS header/library mismatch link check function */
+#define LALINFERENCE_VCS_LINK_CHECK LALINFERENCE_CLEAN_cbb23a10b4a59e71d483d5d1eefbe78dfbee4b5f_VCS_HEADER_LIBRARY_MISMATCH
+void LALINFERENCE_VCS_LINK_CHECK(void);
 
 #ifdef __cplusplus
 }
@@ -72,7 +56,3 @@ static const LALVCSInfo lalInferenceVCSInfoHeader = { \
 #endif /* _LALINFERENCEVCSINFOHEADER_H */
 
 /** \endcond */
-
-/*
- * vim: tw=0 ts=2 et
- */
